
#ifndef Tata_h
#define Tata_h
#include <inttypes.h>

#define ASCII 0
#define FIRMATA 2
#define SYSEX 1

#include "Boards.h"




// We hope to fit at least 64 bytes of data
#define TATA_BUFFER_SIZE 68
#define TATA_BUFFER_SAFE_NULL 67 // NOTHING SHOULD BE ABLE TO WRITE IN THE SAFE NULL
#define TATA_BUFFER_LAST_INDEX 66



extern "C" {
// callback function
    typedef void (*TataCallbackFunction)(void);
}


class Tata_
{

public:
 Tata_(void);
 void begin(long baud);
 void begin(long baud, TataCallbackFunction newFunction,char* string);
  int readInt();
  long readLong(); // Added based on a suggestion by G. Paolo Sani
  char readChar();
  void copyString(char *string, uint8_t size);
  uint8_t checkString(char *string);
  void process();
  uint8_t available(); // Returns the number of available words of an ASCII message
 
  uint8_t sysexRead();
  uint8_t sysexAvailable(); // Returns the number of available 14bit bytes of a SYSEX message
  
private:
  void nextWord();
  void clear();
  void doTheASCII();
  void doTheFIRMATA();
  void printInt(int data);
  void printUnknown();
  void sendDigitalMessage(uint8_t port, uint8_t value);
  uint8_t writePort(uint8_t port, uint8_t value, uint8_t bitmask);
  uint8_t readPort(uint8_t port);
  //uint8_t sendDigitalPort(uint8_t port, uint8_t value);
  void printVersion();
  void printVersionFirmata();
  void reset();
  
  
  
  char* description;
  
  uint8_t message_ready;
  uint8_t message_type;
  TataCallbackFunction callback;
  
  // Reporting
  unsigned long  millisToWait;
  
  uint8_t digitalReporting;
  uint8_t analogReporting;
  
  uint8_t reportDigitalIndex;
  uint8_t reportDigital[TOTAL_PORTS];   
  uint8_t reportDigitalPrevious[TOTAL_PORTS];
  
  uint8_t reportAnalogIndex;
  uint8_t reportAnalog[TOTAL_ANALOG_PORTS];
  
  

  // Data buffer
  uint8_t bufferIndex; // Index where to write the data. 
  char buffer[TATA_BUFFER_SIZE]; // Buffer that holds the data
  // msg_length in FIRMATA: Number of bytes expected to finish a Firmata message
  // msg_length in SYSEX: Number of values in a finished message
  // msg_length in ASCII: Number of bytes in a finished message
  uint8_t msg_length;
  // current in FIRMATA: Unused
  // current in SYSEX: current value
  // current in ASCII: current word
  uint8_t current;
  
  // SYSEX ONLY
  uint8_t sysex_MSB; //Flag that indicates if the next sysex byte is the MSB or not

  
  // ASCII ONLY
  uint8_t word_count; //The number of words in a finished message
   uint8_t valid_char;  //Flag that indicates if at least one valid char has been added to a message

  
  
};

extern Tata_ Tata;


#endif

